#   
#   @file   Makefile.inc
#
#   @brief  Base makefile for building the user-side of SysLink Library 
#           and Sample Application.
#
#
#   ============================================================================
#
#   Copyright (c) 2008-2009, Texas Instruments Incorporated
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#   
#   *  Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#   
#   *  Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#   
#   *  Neither the name of Texas Instruments Incorporated nor the names of
#      its contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#   
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
#   OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#   OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#   EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#   Contact information for paper mail:
#   Texas Instruments
#   Post Office Box 655303
#   Dallas, Texas 75265
#   Contact information: 
#   http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
#   DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
#   ============================================================================
#   

# ---------------------------------------------------------------------------- #
# Defines: App build can override.                                             #
# ---------------------------------------------------------------------------- #
# Platform defines
ifeq ("$(SYSLINK_PLATFORM)", "")
# Default value
SYSLINK_PLATFORM := TI81XX
endif # ifeq ("$(SYSLINK_PLATFORM)", "")

ifeq ("$(SYSLINK_PLATFORM)", "OMAP3530")
COMPILE_FLAGS += -DSYSLINK_PLATFORM_OMAP3530
endif # ifeq ("$(SYSLINK_PLATFORM)", "OMAP3530")

ifeq ("$(SYSLINK_PLATFORM)", "TI81XX")
COMPILE_FLAGS += -DSYSLINK_PLATFORM_TI81XX

#If being built for EZSDK, this define needs to be provided to the build
#system.
ifeq ("$(SYSLINK_SDK)", "EZSDK")
COMPILE_FLAGS += -DSYSLINK_SDK_EZSDK
endif # ifeq ("$(SYSLINK_SDK)", "EZSDK")

#COMPILE_FLAGS += -DTI81XX_DSP
COMPILE_FLAGS += -DTI81XX_VIDEOM3
#COMPILE_FLAGS += -DTI81XX_VPSSM3
endif # ifeq ("$(SYSLINK_PLATFORM)", "TI81XX")

ifeq ("$(SYSLINK_PLATFORM)", "OMAPL1XX")
COMPILE_FLAGS += -DSYSLINK_PLATFORM_OMAPL1XX
endif # ifeq ("$(SYSLINK_PLATFORM)", "OMAPL1XX")

ifeq ("$(SYSLINK_PLATFORM)", "C6474")
COMPILE_FLAGS += -DSYSLINK_PLATFORM_C6474 -DLINUX_THREAD
COMPILE_FLAGS += -DSYSLINK_NOT_USE_FRAMEQ -DSYSLINK_NOT_USE_RINGIO
GCC_TOOL_WRAPPER := 1
endif # ifeq ("$(SYSLINK_PLATFORM)", "C6474")

ifeq ("$(SYSLINK_PLATFORM)", "C6472")
COMPILE_FLAGS += -DSYSLINK_PLATFORM_C6472 -DLINUX_THREAD
COMPILE_FLAGS += -DSYSLINK_NOT_USE_FRAMEQ -DSYSLINK_NOT_USE_RINGIO
GCC_TOOL_WRAPPER := 1
endif # ifeq ("$(SYSLINK_PLATFORM)", "C6472")

# Operating System
ifeq ("$(SYSLINK_BUILDOS_LINUX)", "")
SYSLINK_BUILDOS_LINUX := 1
endif # ifeq ("$(SYSLINK_BUILDOS_LINUX)", "")
ifeq ("$(SYSLINK_BUILDOS_LINUX)", "1")
COMPILE_FLAGS += -DSYSLINK_BUILDOS_LINUX
endif # ifeq ("$(SYSLINK_BUILDOS_LINUX)", "1")

# Optimize build flag
ifeq ("$(SYSLINK_BUILD_OPTIMIZE)", "")
SYSLINK_BUILD_OPTIMIZE :=
endif # ifeq ("$(SYSLINK_BUILD_OPTIMIZE)", "")
ifeq ("$(SYSLINK_BUILD_OPTIMIZE)", "1")
COMPILE_FLAGS += -DSYSLINK_BUILD_OPTIMIZE
endif # ifeq ("$(SYSLINK_BUILD_OPTIMIZE)", "1")

# Debug build
ifeq ("$(SYSLINK_BUILD_DEBUG)", "")
SYSLINK_BUILD_DEBUG := 1
endif # ifeq ("$(SYSLINK_BUILD_DEBUG)", "")
ifeq ("$(SYSLINK_BUILD_DEBUG)", "1")
COMPILE_FLAGS += -DSYSLINK_BUILD_DEBUG
endif # ifeq ("$(SYSLINK_BUILD_DEBUG)", "1")

# Trace enable
ifeq ("$(SYSLINK_TRACE_ENABLE)", "")
SYSLINK_TRACE_ENABLE := 1
endif # ifeq ("$(SYSLINK_TRACE_ENABLE)", "")
ifeq ("$(SYSLINK_TRACE_ENABLE)", "1")
COMPILE_FLAGS += -DSYSLINK_TRACE_ENABLE
endif # ifeq ("$(SYSLINK_TRACE_ENABLE)", "1")

# For Netra simulator
ifeq ("$(SYSLINK_PLATFORM)", "TI81XX")
#COMPILE_FLAGS += -DNETRA_SIMULATOR    # Build is for Netra simulator
endif # ifeq ("$(SYSLINK_PLATFORM)", "TI81XX")

COMPILE_FLAGS += -DSYSLINK_BUILD_HLOS
# ---------------------------------------------------------------------------- #
# Toolchain dir: App build can override.                                       #
# ---------------------------------------------------------------------------- #
ifeq ("$(TOOLCHAIN_PREFIX)", "")
ifeq ("$(SYSLINK_PLATFORM)", "OMAP3530")
TOOLCHAIN_PREFIX := /opt/DSPLINK/arm-2009q1-203/bin/arm-none-linux-gnueabi-
endif # ifeq ("$(SYSLINK_PLATFORM)", "OMAP3530")

ifeq ("$(SYSLINK_PLATFORM)", "TI81XX")
TOOLCHAIN_PREFIX := /opt/DSPLINK/arm-2009q1-203/bin/arm-none-linux-gnueabi-
endif # ifeq ("$(SYSLINK_PLATFORM)", "TI81XX")

ifeq ("$(SYSLINK_PLATFORM)", "OMAPL1XX")
TOOLCHAIN_PREFIX := /opt/DSPLINK/arm-2009q1-203/bin/arm-none-linux-gnueabi-
endif # ifeq ("$(SYSLINK_PLATFORM)", "OMAPL1XX")

ifeq ("$(SYSLINK_PLATFORM)", "C6472")
TOOLCHAIN_PREFIX := /opt/DSPLINK/arm-2009q1-203/ecn_linux_c6x/sdk/bin/c6x-coff-linux-
endif # ifeq ("$(SYSLINK_PLATFORM)", "C6472")

ifeq ("$(SYSLINK_PLATFORM)", "C6474")
TOOLCHAIN_PREFIX := /opt/DSPLINK/arm-2009q1-203/ecn_linux_c6x/sdk/bin/c6x-coff-linux-
endif # ifeq ("$(SYSLINK_PLATFORM)", "C6474")

endif # ifeq ("$(TOOLCHAIN_PREFIX)", "")


# ---------------------------------------------------------------------------- #
# Flags                                                                        #
# ---------------------------------------------------------------------------- #
CC               := $(TOOLCHAIN_PREFIX)gcc
LD               := $(TOOLCHAIN_PREFIX)ld
AR               := $(TOOLCHAIN_PREFIX)ar

CFLAGS           += -Wall -ffloat-store -D_REENTRANT
COPTIMIZE        := -O3

ifeq ("$(GCC_TOOL_WRAPPER)", "1")
ifeq ("$(TOOLCHAIN_PREFIX)", "c6x-coff-linux-")
LINKFLAGS        := -Wl,-ar -pthread 
else
LINKFLAGS        :=  --static -dsbt -pthread 
CFLAGS           += -gcc -dsbt -static
endif # ifeq ("$(TOOLCHAIN_PREFIX)", "c6x-coff-linux-")
else
LINKFLAGS        := -lpthread -lc
endif # ifeq ("$(GCC_TOOL_WRAPPER)", "1")

# ---------------------------------------------------------------------------- #
# Source files                                                                 #
# ---------------------------------------------------------------------------- #
# IPC
OBJECT :=
-include $(SYSLINK_ROOT)/ti/syslink/ipc/hlos/Makefile.inc
-include $(SYSLINK_ROOT)/ti/syslink/ipc/hlos/usr/Makefile.inc
-include $(SYSLINK_ROOT)/ti/syslink/ipc/hlos/usr/Linux/Makefile.inc
# Utility
-include $(SYSLINK_ROOT)/ti/syslink/utils/hlos/Makefile.inc
-include $(SYSLINK_ROOT)/ti/syslink/utils/common/Makefile.inc
-include $(SYSLINK_ROOT)/ti/syslink/utils/hlos/usr/Makefile.inc
-include $(SYSLINK_ROOT)/ti/syslink/utils/hlos/usr/Linux/Makefile.inc
-include $(SYSLINK_ROOT)/ti/syslink/utils/hlos/usr/osal/Linux/Makefile.inc
# Proc Manager
-include $(SYSLINK_ROOT)/ti/syslink/procMgr/hlos/usr/Makefile.inc
-include $(SYSLINK_ROOT)/ti/syslink/procMgr/hlos/usr/Linux/Makefile.inc
#Syslink configuration
-include $(SYSLINK_ROOT)/ti/syslink/cfg/Makefile.inc

# ---------------------------------------------------------------------------- #
# Header files                                                                 #
# Change the relative path enviorment according to sample directory location   #
# ---------------------------------------------------------------------------- #
ifeq ("$(IPC_DIR)", "")
IPC_DIR := /db/psp_git/tools/ipc/ipc_1_22_05_27/packages
endif
COMPILE_FLAGS += -I$(IPC_DIR)
COMPILE_FLAGS += -I$(SYSLINK_ROOT)
COMPILE_FLAGS += -I$(SYSLINK_ROOT)/ti/syslink/inc
COMPILE_FLAGS += -I$(SYSLINK_ROOT)/ti/syslink/inc/usr
COMPILE_FLAGS += -I$(SYSLINK_ROOT)/ti/syslink/inc/usr/Linux
COMPILE_FLAGS += -I$(SYSLINK_ROOT)/ti/syslink/samples/hlos
COMPILE_FLAGS += $(addprefix -I,$(subst ;, ,$(SYSLINK_PKGPATH)))

COBJS       = $(addprefix $(OBJS_DIR)/, $(addsuffix .o, $(notdir $(basename $(CSRCS)))))
PCOBJS      = $(addsuffix profile,  $(COBJS))
DCOBJS      = $(addsuffix debug,    $(COBJS))
RCOBJS      = $(addsuffix release,  $(COBJS))

################################################################################
## Quiet build
################################################################################
ifndef V
Q           := @
endif

################################################################################
## Sample Application Library Dependency
################################################################################
ifdef APP_DEP_LIB
APP_DEP_LIB_PROFILE := $(addsuffix _profile, $(APP_DEP_LIB))
APP_DEP_LIB_DEBUG   := $(addsuffix _debug,   $(APP_DEP_LIB))
APP_DEP_LIB_RELEASE := $(addsuffix _release, $(APP_DEP_LIB))
endif

################################################################################
## SysLink sources Build rules
################################################################################
# Compile template for source files
define Compile_template
# Rule to build standard
$(OBJS_DIR)/$(notdir $(basename $(1)).o): $(basename $(1)).c
	@echo Compiling: $(basename $(1)).c
ifeq ("$(GCC_TOOL_WRAPPER)", "1")
	$(Q)$(CC) $(CFLAGS) $(COPTIMIZE) $(COMPILE_FLAGS) -D DEBUG -c -o $(OBJS_DIR)/$(notdir $(basename $(1)).o) $(basename $(1)).c
else
	$(Q)$(CC) $(CFLAGS) $(COPTIMIZE) $(COMPILE_FLAGS) -ggdb -D DEBUG -c -o $(OBJS_DIR)/$(notdir $(basename $(1)).o) $(basename $(1)).c
endif

# Rule to build profile
$(OBJS_DIR)/$(notdir $(basename $(1)).oprofile): $(basename $(1)).c
	@echo Compiling: $(basename $(1)).c
	$(Q)$(CC) $(CFLAGS) $(COPTIMIZE) $(COMPILE_FLAGS) -pg -ggdb -D DEBUG -c -o $(OBJS_DIR)/$(notdir $(basename $(1)).oprofile) $(basename $(1)).c

# Rule to build debug
$(OBJS_DIR)/$(notdir $(basename $(1)).odebug): $(basename $(1)).c
	@echo Compiling: $(basename $(1)).c
ifeq ("$(GCC_TOOL_WRAPPER)", "1")
	$(Q)$(CC) $(CFLAGS) $(COMPILE_FLAGS) -O0 -D DEBUG -c -o $(OBJS_DIR)/$(notdir $(basename $(1)).odebug) $(basename $(1)).c
else
	$(Q)$(CC) $(CFLAGS) $(COMPILE_FLAGS) -O0 -ggdb -D DEBUG -c -o $(OBJS_DIR)/$(notdir $(basename $(1)).odebug) $(basename $(1)).c
endif

# Rule to build release
$(OBJS_DIR)/$(notdir $(basename $(1)).orelease): $(basename $(1)).c
	@echo Compiling: $(basename $(1)).c
ifeq ("$(GCC_TOOL_WRAPPER)", "1")
	$(Q)$(CC) $(CFLAGS) $(COPTIMIZE) $(COMPILE_FLAGS) -D NDEBUG -c -o $(OBJS_DIR)/$(notdir $(basename $(1)).orelease) $(basename $(1)).c
else
	$(Q)$(CC) $(CFLAGS) $(COPTIMIZE) $(COMPILE_FLAGS) -D NDEBUG -c -o $(OBJS_DIR)/$(notdir $(basename $(1)).orelease) $(basename $(1)).c
endif
endef

# Create rules for compiling C files
$(foreach prog, $(CSRCS),  $(eval $(call Compile_template,$(prog))))

# Rule to build libraries
$(LIB_DIR)/$(SYSLINK_LIB):         $(COBJS)
	@echo Building library $@
	$(Q)$(LD) $(COBJS) -r -o $@
$(LIB_DIR)/$(SYSLINK_LIB)_profile: $(PCOBJS)
	@echo Building library $@
	$(Q)$(LD) $(PCOBJS) -r -o $@
$(LIB_DIR)/$(SYSLINK_LIB)_debug:   $(DCOBJS)
	@echo Building library $@
	$(Q)$(LD) $(DCOBJS) -r -o $@
$(LIB_DIR)/$(SYSLINK_LIB)_release: $(RCOBJS)
	@echo Building library $@
	$(Q)$(LD) $(RCOBJS) -r -o $@

# Rule to build acrhives
$(LIB_DIR)/$(SYSLINK_ARCHIVE):         $(COBJS)
	@echo Archiving $@
	$(Q)$(AR) -rs $@ $(COBJS)
$(LIB_DIR)/$(SYSLINK_ARCHIVE)_profile: $(PCOBJS)
	@echo Archiving $@
	$(Q)$(AR) -rs $@ $(PCOBJS)
$(LIB_DIR)/$(SYSLINK_ARCHIVE)_debug:   $(DCOBJS)
	@echo Archiving $@
	$(Q)$(AR) -rs $@ $(DCOBJS)
$(LIB_DIR)/$(SYSLINK_ARCHIVE)_release: $(RCOBJS)
	@echo Archiving $@
	$(Q)$(AR) -rs $@ $(RCOBJS)

################################################################################
## Samples sources Build rules
################################################################################
SAMPLES_OBJS_DIR = $(SAMPLES_DIR)/$(SAMPLE)
SAMPLE_COBJS   = $(addprefix $(SAMPLES_OBJS_DIR)/, $(addsuffix .o, $(notdir $(basename $(SAMPLE_CSRCS)))))
SAMPLE_PCOBJS  = $(addsuffix profile,  $(SAMPLE_COBJS))
SAMPLE_DCOBJS  = $(addsuffix debug,    $(SAMPLE_COBJS))
SAMPLE_RCOBJS  = $(addsuffix release,  $(SAMPLE_COBJS))
SAMPLE_EXE_COBJS   = $(addprefix $(SAMPLES_OBJS_DIR)/, $(addsuffix .o, $(notdir $(basename $(SAMPLE_EXE_CSRCS)))))
SAMPLE_EXE_PCOBJS  = $(addsuffix profile,  $(SAMPLE_EXE_COBJS))
SAMPLE_EXE_DCOBJS  = $(addsuffix debug,    $(SAMPLE_EXE_COBJS))
SAMPLE_EXE_RCOBJS  = $(addsuffix release,  $(SAMPLE_EXE_COBJS))

# Compile template for sample source files
define SampleCompile_template
# Rule to build standard
$(SAMPLES_OBJS_DIR)/$(notdir $(basename $(1)).o): $(basename $(1)).c
	@echo Compiling: $(basename $(1)).c
ifeq ("$(GCC_TOOL_WRAPPER)", "1")
	$(Q)$(CC) $(CFLAGS) $(COPTIMIZE) $(COMPILE_FLAGS) -D DEBUG -c -o $$@ $$<
else
	$(Q)$(CC) $(CFLAGS) $(COPTIMIZE) $(COMPILE_FLAGS) -ggdb -D DEBUG -c -o $$@ $$<
endif

# Rule to build profile
$(SAMPLES_OBJS_DIR)/$(notdir $(basename $(1)).oprofile): $(basename $(1)).c
	@echo Compiling: $(basename $(1)).c
	$(Q)$(CC) $(CFLAGS) $(COPTIMIZE) $(COMPILE_FLAGS) -pg -ggdb -D DEBUG -c -o $$@ $$<

# Rule to build debug
$(SAMPLES_OBJS_DIR)/$(notdir $(basename $(1)).odebug): $(basename $(1)).c
	@echo Compiling: $(basename $(1)).c
ifeq ("$(GCC_TOOL_WRAPPER)", "1")
	$(Q)$(CC) $(CFLAGS) $(COMPILE_FLAGS) -O0 -D DEBUG -c -o $$@  $$<
else
	$(Q)$(CC) $(CFLAGS) $(COMPILE_FLAGS) -O0 -ggdb -D DEBUG -c -o $$@ $$<
endif

# Rule to build release
$(SAMPLES_OBJS_DIR)/$(notdir $(basename $(1)).orelease): $(basename $(1)).c
	@echo Compiling: $(basename $(1)).c
ifeq ("$(GCC_TOOL_WRAPPER)", "1")
	$(Q)$(CC) $(CFLAGS) $(COPTIMIZE) $(COMPILE_FLAGS) -D NDEBUG -c -o $$@ $$<
else
	$(Q)$(CC) $(CFLAGS) $(COPTIMIZE) $(COMPILE_FLAGS) -D NDEBUG -c -o $$@ $$<
endif
endef

# Create rules for compiling C files
$(foreach prog,$(SAMPLE_CSRCS),$(eval $(call SampleCompile_template,$(prog))))
$(foreach prog,$(SAMPLE_EXE_CSRCS),$(eval $(call SampleCompile_template,$(prog))))

# Rule to build libraries
$(SAMPLES_DIR)/$(APP_LIB):         $(SAMPLE_COBJS)
	@echo Building library $@
	$(Q)$(LD) $(SAMPLE_COBJS) -r -o $@
$(SAMPLES_DIR)/$(APP_LIB)_debug:   $(SAMPLE_DCOBJS)
	@echo Building library $@
	$(Q)$(LD) $(SAMPLE_DCOBJS) -r -o $@
$(SAMPLES_DIR)/$(APP_LIB)_profile: $(SAMPLE_PCOBJS)
	@echo Building library $@
	$(Q)$(LD) $(SAMPLE_PCOBJS) -r -o $@
$(SAMPLES_DIR)/$(APP_LIB)_release: $(SAMPLE_RCOBJS)
	@echo Building library $@
	$(Q)$(LD) $(SAMPLE_RCOBJS) -r -o $@

# Rule to build executables
$(SAMPLES_DIR)/$(SAMPLE).exe: $(SAMPLE_COBJS) $(SAMPLE_EXE_COBJS) $(LIB_DIR)/$(SYSLINK_LIB)
	@echo Building $@
ifeq ("$(GCC_TOOL_WRAPPER)", "1")
	$(Q)$(CC) $(LINKFLAGS)  -Wall -o $@ $(SAMPLE_COBJS)  $(SAMPLE_EXE_COBJS)                        \
	$(LIB_DIR)/$(SYSLINK_ARCHIVE)
else
	$(Q)$(CC) $(LINKFLAGS) -lc -ggdb -Wall -o $@ $(SAMPLE_COBJS) $(SAMPLE_EXE_COBJS)                \
	$(LIB_DIR)/$(SYSLINK_LIB) $(APP_DEP_LIB)
endif
$(SAMPLES_DIR)/$(SAMPLE).exe_debug: $(SAMPLE_DCOBJS) $(SAMPLE_EXE_DCOBJS) $(LIB_DIR)/$(SYSLINK_LIB)_debug
	@echo Building $@
ifeq ("$(GCC_TOOL_WRAPPER)", "1")
	$(Q)$(CC) $(LINKFLAGS) -O0 -Wall -o $@ $(SAMPLE_DCOBJS) $(SAMPLE_EXE_DCOBJS)                    \
	$(LIB_DIR)/$(SYSLINK_ARCHIVE)_debug
else
	$(Q)$(CC) $(LINKFLAGS) -lc -O0 -ggdb -Wall -o $@ $(SAMPLE_DCOBJS) $(SAMPLE_EXE_DCOBJS)          \
	$(LIB_DIR)/$(SYSLINK_LIB)_debug  $(APP_DEP_LIB_DEBUG)
endif
$(SAMPLES_DIR)/$(SAMPLE).exe_profile: $(SAMPLE_PCOBJS) $(SAMPLE_EXE_PCOBJS) $(LIB_DIR)/$(SYSLINK_LIB)_profile
	@echo Building $@
	$(Q)$(CC) $(LINKFLAGS) -lc -pg -ggdb -Wall -o $@ $(SAMPLE_PCOBJS) $(SAMPLE_EXE_PCOBJS)          \
	$(LIB_DIR)/$(SYSLINK_LIB)_profile $(APP_DEP_LIB_PROFILE)
$(SAMPLES_DIR)/$(SAMPLE).exe_release: $(SAMPLE_RCOBJS) $(SAMPLE_EXE_RCOBJS) $(LIB_DIR)/$(SYSLINK_LIB)_release
	@echo Building $@
ifeq ("$(GCC_TOOL_WRAPPER)", "1")
	$(Q)$(CC) $(LINKFLAGS) -O3 -Wall -o $@ $(SAMPLE_RCOBJS) $(SAMPLE_EXE_RCOBJS)                \
	$(LIB_DIR)/$(SYSLINK_ARCHIVE)_release
else
	$(Q)$(CC) $(LINKFLAGS) -lc -O3 -Wall -o $@ $(SAMPLE_RCOBJS) $(SAMPLE_EXE_RCOBJS)                \
	$(LIB_DIR)/$(SYSLINK_LIB)_release $(APP_DEP_LIB_RELEASE)
endif
$(SAMPLES_DIR)/$(SAMPLE).exe_static: $(SAMPLE_RCOBJS) $(SAMPLE_EXE_RCOBJS) $(LIB_DIR)/$(SYSLINK_LIB)_release
	@echo Building $@
	$(Q)$(CC) $(LINKFLAGS) -static -lc -O3 -Wall -o $@ $(SAMPLE_RCOBJS) $(SAMPLE_EXE_RCOBJS)        \
	$(LIB_DIR)/$(SYSLINK_LIB)_release $(APP_DEP_LIB_RELEASE)

# Rule to move executables to bin folder
move:
	@mv $(SAMPLES_DIR)/$(SAMPLE).exe         $(SAMPLES_EXES_DIR)/$(SAMPLE)

move_profile:
	@mv $(SAMPLES_DIR)/$(SAMPLE).exe_profile $(SAMPLES_EXES_DIR)/$(SAMPLE)_profile

move_debug:
	@mv $(SAMPLES_DIR)/$(SAMPLE).exe_debug   $(SAMPLES_EXES_DIR)/$(SAMPLE)_debug

move_release:
	@mv $(SAMPLES_DIR)/$(SAMPLE).exe_release $(SAMPLES_EXES_DIR)/$(SAMPLE)_release
