/*
 *  @file   FrameQBufMgr.c
 *
 *  @brief  Implementation of FrameQBufMgr module on RTOS
 *          side(DSP/BIOS).
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */



/*-------------------------    XDC  specific includes ------------------------*/
#include <xdc/std.h>
#include <xdc/runtime/Memory.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Startup.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/IGateProvider.h>
#include <xdc/runtime/Gate.h>
#include <ti/sysbios/gates/GateSwi.h>


/*-------------------------    Generic includes  -----------------------------*/
#include <string.h>

/*-------------------------    BIOS includes    ------------------------------*/
#include <ti/sysbios/gates/GateMutex.h>

#ifdef xdc_target__isaCompatible_64P
#include <ti/sysbios/family/c64p/Cache.h>
#else
#include <ti/sysbios/hal/Cache.h>
#endif

/*-------------------------    IPC module specific includes ------------------*/
#include <ti/sdo/ipc/Ipc.h>




#include "package/internal/FrameQBufMgr.xdc.h"

#include <ti/syslink/ipc/rtos/interfaces/IFrameQ.h>
#include <ti/syslink/ipc/rtos/interfaces/IFrameQBufMgr.h>

#include <ti/syslink/inc/ClientNotifyMgr.h>

#include <ti/syslink/ipc/rtos/interfaces/_SysLink.h>
//#include <ti/syslink/ipc/rtos/_FrameQ.h> /* For FrameQBufMgr_Frame declarations */
#include <ti/syslink/ipc/rtos/_FrameQBufMgr.h>
#include <ti/syslink/ipc/rtos/shMem/_FrameQBufMgr_ShMem.h>
#include <ti/sdo/ipc/_GateMP.h>
#include <ti/sdo/utils/_NameServer.h>
#include <ti/sdo/ipc/_SharedRegion.h>
#include <ti/sdo/utils/_MultiProc.h>
#include <ti/sdo/ipc/_ListMP.h>

/** ============================================================================
 *  @const  IPC_BUFFER_ALIGN
 *
 *  @desc   Macro to align a number.
 *          x: The number to be aligned
 *          y: The value that the number should be aligned to.
 *  ============================================================================
 */
#define IPC_BUFFER_ALIGN(x, y) (UInt32)((UInt32)((x + y - 1) / y) * y)

#define FRAMEQBUFMGR_NAMESERVERNAME         "FrameQBufMgr"

/*TODO: Move these defines to xdc interface file */
#define FRAMEQBUFMGR_INSTNO_BITS             8u
#define FRAMEQBUFMGR_INSTNO_MASK             0xFF
#define FRAMEQBUFMGR_INSTNO_BITOFFSET        0u
#define FRAMEQBUFMGR_CREATORPROCID_BITS      8u
#define FRAMEQBUFMGR_CREATORPROCID_MASK      0xFF
#define FRAMEQBUFMGR_CREATORPROCID_BITOFFSET 8u

Ptr gFrameQBufMgr_nsHandle;
/* Internal API declarations */
Int32
_FrameQBufMgr_create (FrameQBufMgr_Object* obj,
                      const ti_syslink_ipc_rtos_FrameQBufMgr_Params* params,
                      Error_Block *eb);

Int32
 _FrameQBufMgr_open (FrameQBufMgr_Object* obj,
                     const ti_syslink_ipc_rtos_FrameQBufMgr_Params* params,
                     Error_Block *eb);



/*
 *  ======== FrameQBufMgr_getConfig ========
 *  API to get the module wide config params.
 */
Int32 FrameQBufMgr_getConfig( FrameQBufMgr_Config* cfg)
{
    Int32 status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != cfg), FrameQBufMgr_A_invalidParams);

    if (NULL != cfg) {
        /* (If setup has not yet been called) */
        if (FrameQBufMgr_module->refCount == 0) {
            /* Return the default static config values */
            cfg->eventNo        = FrameQBufMgr_NOTIFY_RESERVED_EVENTNO;
            cfg->useNameServer  = TRUE;
            cfg->maxInstances   = FrameQBufMgr_MAXINSTANCES;
        }
        else {
            /* Return the currently configured parameters */
            memcpy (cfg,
                    &FrameQBufMgr_module->cfg,
                    sizeof (FrameQBufMgr_Config));
        }

    }
    else {
        status = FrameQBufMgr_E_INVALIDARG;
    }
    return (status);
}

/*
 *  ======== FrameQBufMgr_getSharedParams ========
 */
static Void FrameQBufMgr_getSharedParams(FrameQBufMgr_Params *sparams,
    const ti_syslink_ipc_rtos_FrameQBufMgr_Params *params)
{
    sparams->commonCreateParams.size = params->commonCreateParams.size;
    sparams->commonCreateParams.ctrlInterfaceType = (FrameQBufMgr_Interface)
                                       params->commonCreateParams.ctrlInterfaceType;

    sparams->commonCreateParams.name = params->commonCreateParams.name;
    sparams->commonCreateParams.size = params->commonCreateParams.size;

    sparams->impParams         = params->impParams;
    sparams->impParamsSize       = params->impParamsSize;
}

/*
 *  ======== FrameQBufMgr_Params_init ========
 */
Void FrameQBufMgr_Params_init(FrameQBufMgr_Params *sparams)
{
    ti_syslink_ipc_rtos_FrameQBufMgr_Params params;

    ti_syslink_ipc_rtos_FrameQBufMgr_Params_init(&params);
    FrameQBufMgr_getSharedParams(sparams, &params);
}

/*
 *  ======== FrameQBufMgr_create ========
 *  Function to create a FrameQ.params should be of type specific to implementation.
 * For example if apps want to create FrameQ using SharedMemory implementation
 * they have to pass FrameQBufMgr_ShMem_Params to this create call.
 */
FrameQBufMgr_Handle
FrameQBufMgr_create (Ptr params)
{
    Int32 status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Params   *appfqBufMgrParams = (FrameQBufMgr_Params *)params;
    FrameQBufMgr_Handle    fqBufMgrHandle =  NULL;
    FrameQBufMgr_ShMem_Params *pAppFqBufMgrParams;
    ti_syslink_ipc_rtos_FrameQBufMgr_Params fqBufMgrParams;
    ti_syslink_ipc_rtos_shMem_FrameQBufMgr_ShMem_Params shMemParams;
    UInt32 i;

    Error_Block eb;

    Error_init(&eb);
    ti_syslink_ipc_rtos_FrameQBufMgr_Params_init(&fqBufMgrParams);

    memcpy(&fqBufMgrParams.commonCreateParams,
           (Ptr)&(appfqBufMgrParams->commonCreateParams),
           sizeof (ti_syslink_ipc_rtos_FrameQBufMgr_CreateParams));

    /* Initialize the implementation specific params and pass it as impParams*/
    switch (appfqBufMgrParams->commonCreateParams.ctrlInterfaceType)
    {
        case FrameQBufMgr_INTERFACE_SHAREDMEM:
        {
            pAppFqBufMgrParams = (FrameQBufMgr_ShMem_Params *)params;
            ti_syslink_ipc_rtos_shMem_FrameQBufMgr_ShMem_Params_init(
                                                              &shMemParams);


            if (pAppFqBufMgrParams->numFreeFramePools >
                ti_syslink_ipc_rtos_shMem_FrameQBufMgr_ShMem_MAX_POOLS) {
                status = FrameQBufMgr_E_INVALIDARG;
            }
            else if (   pAppFqBufMgrParams->numFreeFramePools
                      > FrameQBufMgr_MAX_POOLS){
                /* Need to increase the MAX_POOLS values in  xdc interfaces file
                 */
                status = FrameQBufMgr_E_INVALIDARG;
            }
            else {
                /* Copy the application  shmem params directly starting fromthe common
                 * create params
                 */
                /* Copy the  common create params */
                memcpy(&(shMemParams.commonCreateParams),
                       &(pAppFqBufMgrParams->commonCreateParams),
                       sizeof (FrameQBufMgr_ShMem_CreateParams));
                /* Copy the rest of the params manually */
                shMemParams.gate                    = (Ptr)
                                             pAppFqBufMgrParams->gate;

                shMemParams.localProtect    =
                    (ti_sdo_ipc_GateMP_LocalProtect)pAppFqBufMgrParams->localProtect;

                shMemParams.remoteProtect   =
                    (ti_sdo_ipc_GateMP_RemoteProtect)pAppFqBufMgrParams->remoteProtect;

                shMemParams.sharedAddr              =
                                             pAppFqBufMgrParams->sharedAddr;
                shMemParams.sharedAddrSize          =
                                             pAppFqBufMgrParams->sharedAddrSize;
                shMemParams.regionId    =
                                       pAppFqBufMgrParams->regionId;

                shMemParams.headerInterfaceType    =
                                       pAppFqBufMgrParams->headerInterfaceType;
                shMemParams.bufInterfaceType    =
                                       pAppFqBufMgrParams->bufInterfaceType;


                shMemParams.sharedAddrHdrBuf        =
                                           pAppFqBufMgrParams->sharedAddrHdrBuf;
                shMemParams.sharedAddrHdrBufSize    =
                                       pAppFqBufMgrParams->sharedAddrHdrBufSize;

                shMemParams.hdrBuf_regionId    =
                                       pAppFqBufMgrParams->hdrBuf_regionId;

                shMemParams.sharedAddrFrmBuf        =
                                           pAppFqBufMgrParams->sharedAddrFrmBuf;
                shMemParams.sharedAddrFrmBufSize    =
                                       pAppFqBufMgrParams->sharedAddrFrmBufSize;

                shMemParams.frmBuf_regionId    =
                                       pAppFqBufMgrParams->frmBuf_regionId;

                shMemParams.numFreeFramePools       =
                                          pAppFqBufMgrParams->numFreeFramePools;


                shMemParams.cpuAccessFlags    =
                                             pAppFqBufMgrParams->cpuAccessFlags;

                shMemParams.numFreeFramesInPool     =
                                        pAppFqBufMgrParams->numFreeFramesInPool;
                shMemParams.numFreeHeaderBufsInPool =
                                    pAppFqBufMgrParams->numFreeHeaderBufsInPool;
                shMemParams.frameHeaderBufSize      =
                                         pAppFqBufMgrParams->frameHeaderBufSize;
                shMemParams.numFrameBufsInFrame     =
                                        pAppFqBufMgrParams->numFrameBufsInFrame;
                shMemParams.numNotifyEntries     =
                                        pAppFqBufMgrParams->numNotifyEntries;

                for (i = 0; i < pAppFqBufMgrParams->numFreeFramePools; i++) {
                    /* As the lay out of elements is same  in both the FrameBufParams
                     * structures copying the pointer directly.
                     */
                    shMemParams.frameBufParams[i] = (ti_syslink_ipc_rtos_shMem_FrameQBufMgr_ShMem_FrameBufParams*)
                                           pAppFqBufMgrParams->frameBufParams[i];
                }

                shMemParams.commonCreateParams.size =
                    sizeof(ti_syslink_ipc_rtos_shMem_FrameQBufMgr_ShMem_Params);
                fqBufMgrParams.impParams = &shMemParams;
                fqBufMgrParams.impParamsSize =
                          sizeof(ti_syslink_ipc_rtos_shMem_FrameQBufMgr_ShMem_Params);
            }
        }
        break;

        default:
            status = FrameQBufMgr_E_FAIL;
    }

    if (status >= 0) {
        /* Nothing to be done for this function. */
        fqBufMgrHandle = ti_syslink_ipc_rtos_FrameQBufMgr_create(
                                                               &fqBufMgrParams,
                                                               &eb);
        if (fqBufMgrHandle == NULL) {
            Error_print(&eb);
            Error_init(&eb);
        }
    }

    return (fqBufMgrHandle);
}

/*
 *  ======== FrameQBufMgr_delete ========
 *  Function to delete the created FrameQ instance.
 */
Int32
FrameQBufMgr_delete (FrameQBufMgr_Handle * handle)
{
    /* Nothing to be done for this function. */
    ti_syslink_ipc_rtos_FrameQBufMgr_delete(handle);

    return(0);
}

/* -------- per-instance functions -------- */
/*
 *  ======== FrameQBufMgr_Instance_init ========
 *  API to create/open the instance.Called internally by
 *  the FrameQBufMgr_create call.
 */
Int
FrameQBufMgr_Instance_init(FrameQBufMgr_Object* obj,
                            const ti_syslink_ipc_rtos_FrameQBufMgr_Params* params,
                            Error_Block *eb)
{
    Int32                         status         = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue (((NULL != obj) && (NULL != params)),
                     FrameQBufMgr_A_invalidParams);
    if (params == NULL) {
        status = FrameQBufMgr_E_INVALIDARG;
        Error_raise(eb,
                    FrameQBufMgr_E_InvalidParams,
                    NULL,
                    0);
    }
    else if (params->commonCreateParams.openFlag == TRUE) {
       status = _FrameQBufMgr_open (obj, params,eb);
       if (status < 0) {
            status = FrameQBufMgr_E_FAIL;
            Error_raise(eb,
                        FrameQBufMgr_E_FrameQBufMgrOpenFail,
                        NULL,
                        0);
       }
    }
    else if (params->commonCreateParams.openFlag == FALSE) {
        status = _FrameQBufMgr_create (obj, params,eb);
       if (status < 0) {
            status = FrameQBufMgr_E_FAIL;
            Error_raise(eb,
                        FrameQBufMgr_E_FrameQBufMgrCreateFail,
                        NULL,
                        0);
       }
    }
    return (status);
}

/*
 *  ======== _FrameQBufMgr_create ========
 * This internal function is called by FrameQBufMgr_Instance_init function when
 * openFlag is  FALSE.
 */
Int32
_FrameQBufMgr_create (FrameQBufMgr_Object* obj,
                      const ti_syslink_ipc_rtos_FrameQBufMgr_Params* params,
                      Error_Block *eb)
{
    Int32                         status    = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_NameServerEntry  entry;
    FrameQBufMgr_ShMem_Handle     shMemHandle;
    UInt32                        len;

    Assert_isTrue ((NULL != obj),
                 FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != params),
                 FrameQBufMgr_A_invalidParams);

    if ( (NULL == obj) || ( NULL == params)) {
        status = FrameQBufMgr_E_INVALIDARG;
        Error_raise(eb,
                    FrameQBufMgr_E_InvalidParams,
                    NULL,
                    0);
    }
    else {
        /* Check if name is already exists in the name server */
        if (FrameQBufMgr_module->nameServer != NULL) {
            len = sizeof (FrameQBufMgr_NameServerEntry);
            status = NameServer_get ((NameServer_Handle)FrameQBufMgr_module->nameServer,
                                     (String)params->commonCreateParams.name,
                                     &entry,
                                     &len,
                                     NULL);
            if (status >= 0) {
                /* Already instance with the name exists */
                status = FrameQBufMgr_E_INST_EXISTS;
                Error_raise(eb,
                            FrameQBufMgr_E_InstanceExists,
                            NULL,
                            0);
            }
            else if (status == NameServer_E_FAIL) {
                /* Error happened in NameServer. Pass the error up. */
                status = FrameQBufMgr_E_FAIL;
                Error_raise(eb,
                            FrameQBufMgr_E_InvalidParams,
                            NULL,
                            0);
            }
            else {
                /* Go ahead and create as there is no instance with that name
                 * exists.
                 */
                 Error_init(eb);
                 status = FrameQBufMgr_S_SUCCESS;
            }
        }
    }

    if (status >= 0) {
        /* Based on the type of interface  mentioned  call create of specific
         * implementation.Currently it has support for FrameQ on shared memory.
         * Need to extend this switch case  to add support for other
         * implementations.
         */
        switch (params->commonCreateParams.ctrlInterfaceType) {
            case FrameQBufMgr_INTERFACE_SHAREDMEM:
            {
                if(FrameQBufMgr_isSupportedBufInterface(params->commonCreateParams.ctrlInterfaceType,
                                                             ((ti_syslink_ipc_rtos_shMem_FrameQBufMgr_ShMem_Params*)
                                                        params->impParams)->bufInterfaceType)) {

                    /* Create the shared memory based FrameQ  instance.*/
                    shMemHandle = FrameQBufMgr_ShMem_create(
                          (ti_syslink_ipc_rtos_shMem_FrameQBufMgr_ShMem_Params*)
                                                            params->impParams,
                                                            eb);
                    if (shMemHandle == NULL) {
                        Error_raise(eb,
                                    FrameQBufMgr_E_FrameQBufMgrCreateFail,
                                    NULL,
                                    0);
                    }
                    else {
                        /* Upcast the handle so that we can call Implementation
                         * specific API using IFrameQBufMgr.
                         */
                        obj->handle        =
                                      FrameQBufMgr_ShMem_Handle_upCast(shMemHandle);
                        obj->ctrlInterfaceType = params->commonCreateParams.ctrlInterfaceType;

                        obj->objType       = FrameQBufMgr_DYNAMIC_CREATE;
                        /*  Get the instId  to update the toplevel object */
                        obj->instId        = FrameQBufMgr_ShMem_getId(shMemHandle);
                        obj->refCount      = 1;
                        obj->notifyId = (UInt32)-1;
                   }
                }
                else {
                    /* Invalid buffer interface type */
                    status = FrameQBufMgr_E_INVALID_INTERFACE;
                    Error_raise(eb,
                                FrameQBufMgr_E_InvalidInterfaceType,
                                NULL,
                                0);                }
            }
            break;

            default:
                status = FrameQBufMgr_E_INVALID_INTERFACE;
                Error_raise(eb,
                            FrameQBufMgr_E_InvalidInterfaceType,
                            NULL,
                            0);
        }
    }

    return (status);
}

/*
 *  ======== FrameQBufMgr_open ========
 */
Int32
FrameQBufMgr_open(FrameQBufMgr_Handle     *handlePtr,
                  Ptr                      intOpenParams)
{
    Int32                        status = FrameQBufMgr_S_SUCCESS;
    FrameQBufMgr_Handle          frmQBufMgrHandle = NULL;
    ti_syslink_ipc_rtos_FrameQBufMgr_Params  params;
    FrameQBufMgr_Object         *obj;
    FrameQBufMgr_OpenParams     *openParams =
                                       (FrameQBufMgr_OpenParams *)intOpenParams;
    Ptr                          impHandle = NULL;
    UInt32                       instId = (UInt32)-1;
    FrameQBufMgr_NameServerEntry entry;
    FrameQBufMgr_NameServerEntry *pentry;
    UInt32                       interfaceType;
    UInt32                       bufInterfaceType;
    UInt8                        nameLen;
    Error_Block                  eb;
    Int32                        key;
    Int32                        index;
    UInt32                       len;

    Assert_isTrue ((NULL != handlePtr), FrameQBufMgr_A_invalidParams);

    /* Reject  if both name and shared Addr is not known */
    if (   (strlen((String)openParams->commonOpenParams.name) == 0)
        && (openParams->commonOpenParams.sharedAddr == NULL)) {
        status = FrameQBufMgr_E_INVALIDARG;
    }
    else {
        Error_init(&eb);
        nameLen = strlen((String)openParams->commonOpenParams.name);

        if (   (nameLen == 0)
            && (openParams->commonOpenParams.sharedAddr == NULL)) {
                status = FrameQBufMgr_E_INVALIDARG;
        }
        else if (   ( (nameLen > 0) && ( nameLen <= (FrameQBufMgr_MAX_NAMELEN-1))
            && (NULL != FrameQBufMgr_module->nameServer))) {
            len = sizeof (FrameQBufMgr_NameServerEntry);
            status = NameServer_get ((NameServer_Handle)FrameQBufMgr_module->nameServer,
                                     (String)openParams->commonOpenParams.name,
                                     &entry,
                                     &len ,
                                     NULL);
            if (status < 0) {
                status = FrameQBufMgr_E_NOTFOUND;
            }
            else {
                interfaceType = entry.ctrlInterfaceType;
                bufInterfaceType  = entry.bufInterfaceType;

            }
        }
        else if (openParams->commonOpenParams.sharedAddr != NULL) {
            pentry = (FrameQBufMgr_NameServerEntry*)
                                       openParams->commonOpenParams.sharedAddr;
            /* Assert that sharedAddr is cache aligned */
            index = SharedRegion_getId(pentry);

            /* Assert that the buffer is in a valid shared region */
            Assert_isTrue(index != SharedRegion_INVALIDREGIONID,
                          Ipc_A_addrNotInSharedRegion);

            Assert_isTrue(((UInt32)pentry %
                          SharedRegion_getCacheLineSize(index) == 0),
                          Ipc_A_addrNotCacheAligned);

            if (SharedRegion_isCacheEnabled(index))  {
                /* Invalidating the entry located in shared memory. The memory
                 * provided by this entry should be in cache aligend size
                 */
                key = Gate_enterSystem();
                Cache_inv (pentry,
                           sizeof (FrameQBufMgr_NameServerEntry),
                           Cache_Type_ALL,
                           TRUE);
                Gate_leaveSystem (key);
            }
            interfaceType = pentry->ctrlInterfaceType;
            bufInterfaceType  = pentry->bufInterfaceType;

        }
        else {
            status = FrameQBufMgr_E_INVALIDARG;
        }

        if (status >= 0) {
             /* Check the interface type and if it is not supported
              * return failure.
              */
            if( !FrameQBufMgr_isSupportedInterface(interfaceType)) {
                status = FrameQBufMgr_E_INVALID_INTERFACE;
                Error_raise(&eb,
                            FrameQBufMgr_E_InvalidInterfaceType,
                            NULL,
                            0);
            }
            else if(!FrameQBufMgr_isSupportedBufInterface(interfaceType,
                                                          bufInterfaceType)) {
                /*! @retval FrameQBufMgr_E_INVALID_INTERFACE
                 * Unsupported interface type.
                 */
                status = FrameQBufMgr_E_INVALID_INTERFACE;
                Error_raise(&eb,
                            FrameQBufMgr_E_InvalidInterfaceType,
                            NULL,
                            0);
            }
            else {
                /* Get the default params */
                ti_syslink_ipc_rtos_FrameQBufMgr_Params_init (&params);
                /* Name is string pointer in CreateParams*/
                params.commonCreateParams.name = (String)
                                             openParams->commonOpenParams.name;
                /* Opening the instance  */
                params.commonCreateParams.openFlag = TRUE;
                /* Call RTSC's create call */
                frmQBufMgrHandle = ti_syslink_ipc_rtos_FrameQBufMgr_create (
                                                                  &params, &eb);
                if (frmQBufMgrHandle == NULL) {
                    status = FrameQBufMgr_E_FAIL;
                }
                else {
                    /* Open the handle to the implementation specific handle
                     * and plug in to the top level FrameQ object.
                     */
                    switch (interfaceType) {
                        case FrameQBufMgr_INTERFACE_SHAREDMEM:
                        {
                            /* Call the RTSC's  open call. Since openparams are
                             * same lay out.Typecasting to
                             *ti_syslink_ipc_rtos_shMem_FrameQBufMgr_ShMem_OpenParams
                             * is ok
                             */
                            status = FrameQBufMgr_ShMem_open(
                                    (FrameQBufMgr_ShMem_Handle *)&impHandle,
                                     (ti_syslink_ipc_rtos_shMem_FrameQBufMgr_ShMem_OpenParams *)openParams);
                            if ((status >= 0) && (impHandle != NULL)) {
                                instId =
                                        FrameQBufMgr_ShMem_getId(impHandle);
                                impHandle =
                                      FrameQBufMgr_ShMem_Handle_upCast(
                                                                     impHandle);
                            }
                            else {
                                FrameQBufMgr_close(&frmQBufMgrHandle);
                            }
                        }
                        break;

                        default:
                            /* NameServer entry does not contain the
                             * implementation type.
                             */
                            status = FrameQBufMgr_E_INVALID_INTERFACE;
                    }
                }
            }
        }
    }

    if (status >= 0) {
        obj                = frmQBufMgrHandle;
        obj->handle        = impHandle;
        obj->ctrlInterfaceType = interfaceType;
        obj->objType       = FrameQBufMgr_DYNAMIC_OPEN;
        obj->instId        = instId;
        *handlePtr         = frmQBufMgrHandle;
    }
    else {
       *handlePtr         = NULL;
    }

    return (status);
}

/*
 *  ======== _FrameQBufMgr_open ========
 * This internal function is called by FrameQBufMgr_Instance_init function when
 * openFlag is  TRUE.
 */
Int32
 _FrameQBufMgr_open (FrameQBufMgr_Object* obj,
                     const ti_syslink_ipc_rtos_FrameQBufMgr_Params* params,
                     Error_Block *eb)
{

    Int32                status = FrameQBufMgr_S_SUCCESS;
    Assert_isTrue ((NULL != obj),
                 FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != params),
                 FrameQBufMgr_A_invalidParams);

    return(status);
}

/*
 *  ======== FrameQBufMgr_Instance_finalize ========
 *  API to finalize the created instance.
 */
 Void
 FrameQBufMgr_Instance_finalize( FrameQBufMgr_Object* obj ,
                                  int status )
 {
    FrameQBufMgr_ShMem_Handle handle;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    switch (obj->objType) {

    case FrameQBufMgr_DYNAMIC_OPEN:
    {
        /* Close the implementation specific handle and delete the top level
         * Object.
         */
        if (obj->handle != NULL) {
            switch (obj->ctrlInterfaceType) {
                case FrameQBufMgr_INTERFACE_SHAREDMEM:
                {
                    handle = FrameQBufMgr_ShMem_Handle_downCast(obj->handle);
                    FrameQBufMgr_ShMem_close(&handle);
                }
                break;

                default:
                break;
            }
        }

    }
    break;

    case FrameQBufMgr_DYNAMIC_CREATE:
    {
        /* Delete the implementation specific handle and delete the top level
         * Object.
         */
        if (obj->handle != NULL) {
            IFrameQBufMgr_delete((IFrameQBufMgr_Handle*)&obj->handle);
        }
        break;

    }

    case FrameQBufMgr_STATIC_CREATE:
    {
        /* TODO: raise error or some assert */
        break;
    }

    case FrameQBufMgr_STATIC_OPEN:
    {
        /* TODO: raise error or some assert */
        break;
    }
    default:
            break;
    }
}

Int32
FrameQBufMgr_close(FrameQBufMgr_Handle *pHandle )
{
    Int32 status = FrameQBufMgr_S_SUCCESS;

    /* Delete the object as  it is last one on this processor */
    FrameQBufMgr_delete (pHandle);

    return (status);
 }

/*
 *  ======== FrameQBufMgr_alloc ========
 *  Function to allocate frame from free frame queue zero .
 */
Int32
FrameQBufMgr_alloc (FrameQBufMgr_Object *obj,
                    FrameQBufMgr_Frame          * frame)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != frame), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_alloc (obj->handle,
                    (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame*)frame);

    return (status);
}

/*
 *  ======== FrameQBufMgr_alloc ========
 *  Function to allocate multiple frames from the FrameQbufMgr instance.
 */
Int32
FrameQBufMgr_allocv (FrameQBufMgr_Object *obj,
                     FrameQBufMgr_Frame  framePtr[],
                     UInt32              freeQId[],
                     UInt8               *numFrames)
{

    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != framePtr), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != freeQId), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != numFrames), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_allocv (obj->handle,
        (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame *)framePtr,
                                   freeQId,
                                   numFrames);

    return (status);
}

/*
 *  ======== FrameQBufMgr_free ========
 *  Function to free frame.
 */
Int32
FrameQBufMgr_free (FrameQBufMgr_Object *obj,
                   FrameQBufMgr_Frame         frame)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != frame), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_free (obj->handle,
                     (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame)frame);

    return (status);
}

/*
 * ======== FrameQBufMgr_freev ========
 * Function to free frames.
 *
 * Function to free multiple frames  to the  FrameQBufMgr.
 */
Int32
FrameQBufMgr_freev (FrameQBufMgr_Object *obj,
                    FrameQBufMgr_Frame         framePtr[],
                    UInt32               numFrames)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != framePtr), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((0 != numFrames), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_freev (obj->handle,
                 (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame *)framePtr,
                                  numFrames);

    return (status);
}

/*
 *  ======== FrameQBufMgr_add ========
 * Function to add  free frames dynamically  which are allocated out side of
 * FrmaeQbufMgr through the  same memory manager.
 */
Ptr
FrameQBufMgr_add (FrameQBufMgr_Object *obj,
                  UInt8                freeQId)
{
    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((FrameQBufMgr_MAX_POOLS > freeQId), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    return (IFrameQBufMgr_add (obj->handle,
                                freeQId));
}
/*
 *  ======== FrameQBufMgr_remove ========
 * Function to remove frame from the FrameQbufMgr.Only allowed for dynamically
 * added frames.
 */
Int32
FrameQBufMgr_remove (FrameQBufMgr_Object *obj,
                     UInt8                freeQId,
                     FrameQBufMgr_Frame         framePtr)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((8 > freeQId), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != framePtr), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_remove (obj->handle,
                                   freeQId,
                                   (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame)framePtr);

    return (status);
}

/*
 *  ======== FrameQBufMgr_registerNotifier ========
 * Function to Register notification with the instance.  It internally registers
 * notification with the individual free queues( free frame pools).
 * If alloc on free queue 0 failed,Notification will be generated if free frames
 * in that queue becomes more than watermark(all the free frames in that queue).
 */
Int32
FrameQBufMgr_registerNotifier (FrameQBufMgr_Object       *obj,
                               FrameQBufMgr_NotifyParams *notifyParams)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != notifyParams), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_registerNotifier (obj->handle,
     (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_NotifyParams*) notifyParams);

    return (status);
}

/* Function to unregister the notification call back function.*/
Int32
FrameQBufMgr_unregisterNotifier (FrameQBufMgr_Object *obj)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_unregisterNotifier (obj->handle);

    return (status);
}

/*
 *  ======== FrameQBufMgr_dup ========
 * API to duplicate the given frame i.e it increments the corresponding
 * frames 's reference count.It internally allocates  headers for the duped
 * frames.It fails if it is not able to dup all the frames.
 */
Int32
FrameQBufMgr_dup (FrameQBufMgr_Object   *obj,
                  FrameQBufMgr_Frame     framePtr,
                  FrameQBufMgr_Frame     dupedFramePtr[],
                  UInt32                 numDupedFrames)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != framePtr), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((0 != numDupedFrames), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_dup (obj->handle,
                                (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame)framePtr,
                                (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame *)dupedFramePtr,
                                numDupedFrames);
    return (status);
}

/*
 *  ======== FrameQBufMgr_dupv ========
 * API to duplicate the given frames i.e it increments the corresponding
 * frames 's reference count.It internally allocates  headers for the duped
 * frames.It fails if it is not able to dup all the frames.
 */
Int32
FrameQBufMgr_dupv (FrameQBufMgr_Object *obj,
                   FrameQBufMgr_Frame   framePtr[],
                   FrameQBufMgr_Frame  **dupedFramePtr,
                   UInt32               numDupedFrames,
                   UInt32               numFrames)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_ShMem_A_invalidParams);

    Assert_isTrue ((NULL != framePtr), FrameQBufMgr_ShMem_A_invalidParams);

    status = IFrameQBufMgr_dupv (obj->handle,
                                 (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame *)framePtr,
                                 (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame **)dupedFramePtr,
                                 numDupedFrames,
                                 numFrames);

    return (status);
}

/*
 *  ======== FrameQBufMgr_writeBack ========
 * Write back the contents  of a frame including frame buffers
 */
Int32
FrameQBufMgr_writeBack(FrameQBufMgr_Object *obj, FrameQBufMgr_Frame frame)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != frame), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_writeBack (obj->handle,
                                      (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame)frame);
    return (status);

}

/*
 *  ======== FrameQBufMgr_invalidate ========
 * Invalidate the contents  of a frame including frame buffers.
 */
Int32
FrameQBufMgr_invalidate(FrameQBufMgr_Object *obj, FrameQBufMgr_Frame frame)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != frame), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_invalidate (obj->handle,
                                       (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame)frame);

    return (status);

}

/*
 *  ======== FrameQBufMgr_writeBackHeaderBuf ========
 * Write back the contents  of buffer . buffe is treated as frame  buffer.
 * bufNumInFrame denotes  buffer number in the frame.This is  to identify
 * the cache flags and cpu access flags for the  buffer.
 */
Int32
FrameQBufMgr_writeBackHeaderBuf(FrameQBufMgr_Object *obj,
                                Ptr headerBuf)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != headerBuf), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_writeBackHeaderBuf (obj->handle, headerBuf);

    return (status);

}

/*
 *  ======== FrameQBufMgr_invalidateHeaderBuf ========
 * Invalidate the contents  of frame buffer .
 *
 * bufIndexInFrame denotes which buffer it is  to identify the cache flags and
 * cpu access flags for the  buffer. This is the sequence number of frame buffer
 * in the frame.
 */
Int32
FrameQBufMgr_invalidateHeaderBuf(FrameQBufMgr_Object *obj,
                                Ptr headerBuf)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != headerBuf), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_invalidateHeaderBuf (obj->handle, headerBuf);

    return (status);

}

/*
 *  ======== FrameQBufMgr_writeBackFrameBuf ========
 * Write back the contents  of buffer . buffe is treated as frame  buffer.
 * bufNumInFrame denotes  buffer number in the frame.This is  to identify
 * the cache flags and cpu access flags for the  buffer.
 */
Int32
FrameQBufMgr_writeBackFrameBuf(FrameQBufMgr_Object *obj,
                               Ptr                  frameBuf,
                               UInt32               size,
                               UInt8                bufNoInFrame)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != frameBuf), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((8 > bufNoInFrame), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_writeBackFrameBuf (obj->handle,
                                              frameBuf,
                                              size,
                                              bufNoInFrame);
    return (status);
}

/*
 *  ======== FrameQBufMgr_ShMem_invalidateFrameBuf ========
 * Invalidate the contents  of frame buffer .
 *
 * bufNumInFrame denotes which buffer it is  to identify the cache flags and
 * cpu access flags for the  buffer. This is the sequence number of frame buffer
 * in the frame.
 */
Int32
FrameQBufMgr_invalidateFrameBuf(FrameQBufMgr_Object *obj,
                                Ptr                  frameBuf,
                                UInt32               size,
                                UInt8                bufNoInFrame)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != frameBuf), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((8 > bufNoInFrame), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_invalidateFrameBuf (obj->handle,
                                               frameBuf,
                                               size,
                                               bufNoInFrame);
    return (status);
}
/*
 *  ======== FrameQBufMgr_translateAddr ========
 * Function to translate address betweem different types.
 * see also FrameQBufMgr_AddrType
 */
Int32
FrameQBufMgr_translateAddr (FrameQBufMgr_Object           *obj,
                            Ptr *                          dstAddr,
                            FrameQBufMgr_AddrType          dstAddrType,
                            Ptr                            srcAddr,
                            FrameQBufMgr_AddrType          srcAddrType,
                            FrameQBufMgr_BufType           bufType)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != dstAddr), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != srcAddr), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_translateAddr (obj->handle,
                                          dstAddr,
                                          (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_AddrType)dstAddrType,
                                          srcAddr,
                                          (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_AddrType)srcAddrType,
                                          (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_BufType)bufType);

    return (status);
}

Int32 ti_syslink_ipc_rtos_FrameQBufMgr_translateAddr__F (FrameQBufMgr_Object *obj,
                            Ptr *                          dstAddr,
                            FrameQBufMgr_AddrType          dstAddrType,
                            Ptr                            srcAddr,
                            FrameQBufMgr_AddrType          srcAddrType,
                            FrameQBufMgr_BufType           bufType)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != dstAddr), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != srcAddr), FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != obj->handle), FrameQBufMgr_A_invalidParams);

    status = IFrameQBufMgr_translateAddr (obj->handle,
                                          dstAddr,
                                          (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_AddrType)dstAddrType,
                                          srcAddr,
                                          (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_AddrType)srcAddrType,
                                          (ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_BufType)bufType);

    return (status);
}
/*
 *  ======== FrameQBufMgr_translateAddr ========
 * Function to get FrameQbufMgr Id.
 */
UInt32
FrameQBufMgr_getId (FrameQBufMgr_Object* obj)
{
    Assert_isTrue ((NULL != obj), FrameQBufMgr_A_invalidParams);

    return(obj->instId);
}

/*
 *  ======== FrameQBufMgr_getHandle ========
 *  Internal API to return the FrameQbufMgr Id.
 */
Ptr
FrameQBufMgr_getHandle (UInt32 Id)
{
    FrameQBufMgr_Handle             handle  = NULL;
    FrameQBufMgr_Handle             nextHandle = NULL;
    UInt32                           i;
    UInt32                           key;
    FrameQBufMgr_Object             *obj;
    UInt16                          instId;
    UInt16                          instNo;
    UInt16                          creProcId;

    instId = Id & 0xFFFF;
    /* Find out the creator procid */
    creProcId = (  (instId >> FRAMEQBUFMGR_CREATORPROCID_BITOFFSET)
                  & FRAMEQBUFMGR_CREATORPROCID_MASK);
    Assert_isTrue ((creProcId < ti_sdo_utils_MultiProc_numProcessors),
                     FrameQBufMgr_A_InvalidCreatorProcId);

    /* Get the instance no of the instance */
    instNo = ( (instId >> FRAMEQBUFMGR_INSTNO_BITOFFSET)
              & FRAMEQBUFMGR_INSTNO_MASK);
    Assert_isTrue ((instNo < FrameQBufMgr_MAXINSTANCES),
                     FrameQBufMgr_A_InvalidInstNo);

    if (instNo < FrameQBufMgr_MAXINSTANCES) {

        key = IGateProvider_enter (FrameQBufMgr_module->gate );
        for ( handle = FrameQBufMgr_Object_first();
              handle != NULL;
              handle = nextHandle) {
              obj =  handle;
               if (obj->instId == instId) {
                    /* Found the handle. return it */
                    break;
               }
              nextHandle = FrameQBufMgr_Object_next(handle) ;
        }

        if (handle == NULL ) {
            for (i = 0; i < FrameQBufMgr_Object_count(); i++) {
               handle = FrameQBufMgr_Object_get(NULL, i);
               obj =  handle;
               if (obj->instId == instId) {
                    /* Found the  handle. return it */
                    break;
               }
            }
        }
        IGateProvider_leave (FrameQBufMgr_module->gate, key );

    }/*if (instNo < FrameQBufMgr_MAXINSTANCES) */

    return ((Ptr)handle);
}

/*
 *  ======== FrameQBufMgr_getNumFreeFrames ========
 *  Function to get the number of free frames available in
 *  FrameQbufMgr's FreeFramePool 0. The returned number free frame may not
 *  exist in the frameQbufMgr  after this call.This is because
 *  if any allocs are done by the other FrameQ writers on the
 *  same FrameQBufMgr.If frees are not other frameQ writers the free Frames
 *  would be even more than the value that we have got.
 *
 */
Int32
FrameQBufMgr_getNumFreeFrames (FrameQBufMgr_Object *obj, UInt32* numFreeFrames)
{
    Int32   status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != numFreeFrames),
                   FrameQBufMgr_A_invalidParams);

    /* Call implementation specific API.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQBufMgr_getNumFreeFrames (obj->handle, numFreeFrames);

    return (status);
}

/*
 *  ======== FrameQBufMgr_getvNumFreeFrames ========
 * Function to get the number of free frames available in the plugged in
 * FrameQbufMgr.
 */
Int32
FrameQBufMgr_getvNumFreeFrames (FrameQBufMgr_Object   * obj,
                                UInt32                  numFreeFrames[],
                                UInt8                   freeFramePoolNo[],
                                UInt8                   numFreeFramePools)
{
    Int32   status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != numFreeFrames),
                   FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((NULL != freeFramePoolNo),
                   FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((0 != numFreeFramePools),
                   FrameQBufMgr_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQBufMgr_getvNumFreeFrames(obj->handle,
                                             numFreeFrames,
                                             freeFramePoolNo,
                                             numFreeFramePools);
    return (status);
}

/*
 *  ======== FrameQBufMgr_isCacheEnabledForHeaderBuf ========
 * Function to find out if cache is enabled for header buffers.
 */
Bool FrameQBufMgr_isCacheEnabledForHeaderBuf(FrameQBufMgr_Object   * obj)
{
    Bool cacheEnabled = FALSE;

    Assert_isTrue ((NULL != obj),
                   FrameQBufMgr_A_invalidParams);

    cacheEnabled =  IFrameQBufMgr_isCacheEnabledForHeaderBuf(obj->handle);

    return (cacheEnabled);
}

/*
 *  ======== FrameQBufMgr_isCacheEnabledForFrameBuf ========
 * Function to find out if cache is enabled for frame buffers.
 */
Bool FrameQBufMgr_isCacheEnabledForFrameBuf(FrameQBufMgr_Object * obj,
                                            UInt8 frameBufIndex)
{
    Bool cacheEnabled = FALSE;

    Assert_isTrue ((NULL != obj),
                   FrameQBufMgr_A_invalidParams);

    Assert_isTrue ((FrameQBufMgr_MAX_FRAMEBUFS > frameBufIndex),
                   FrameQBufMgr_A_invalidParams);

    cacheEnabled =  IFrameQBufMgr_isCacheEnabledForFrameBuf(obj->handle,
                                                            frameBufIndex);

    return (cacheEnabled);
}

/*
 *  ======== FrameQ_control ========
 *  @brief  Provides a hook to perform implementation dependent operation.
 *
 */
Int32
FrameQBufMgr_control (FrameQBufMgr_Object *obj,
                      Int32                cmd,
                      Ptr                  arg)
{
    Int32   status = FrameQBufMgr_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQBufMgr_A_invalidParams);

    /* Call implementation specific API.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQBufMgr_control(obj->handle,
                                   cmd,
                                   arg);
    return (status);
}

/*********************** Internal API ****************************************/
/*
 *  ======== FrameQBufMgr_Module_startup ========
 */
Int FrameQBufMgr_Module_startup(Int phase)
{
    GateSwi_Handle               swiGate;
    Int32                        key;
    Error_Block                  eb;

    if (FrameQBufMgr_module->nameServer != NULL) {
        if (!ti_sdo_utils_NameServer_Module_startupDone()) {
            return (Startup_NOTDONE);
        }
        gFrameQBufMgr_nsHandle = FrameQBufMgr_module->nameServer;
    }

    gFrameQBufMgr_nsHandle = FrameQBufMgr_module->nameServer;

    key = Gate_enterSystem();

    FrameQBufMgr_module->refCount++;

    if(FrameQBufMgr_module->refCount > 1) {
        Gate_leaveSystem(key);
        return (Startup_DONE);
    }
    else {
          if (FrameQBufMgr_module->gate == NULL) {
                /*  Create local SWI gate  for module wide protection
                 * if not created statically.
                 */
                Error_init(&eb);
                swiGate = GateSwi_create(NULL, &eb);
                if (swiGate != NULL) {
                    FrameQBufMgr_module->gate =
                                              GateSwi_Handle_upCast(swiGate);
                    FrameQBufMgr_module->gateType =
                                              FrameQBufMgr_DYNAMIC_CREATE;
                }
                else {
                     /* Release lock */
                    Gate_leaveSystem(key);
                    return (Startup_NOTDONE);
                }
          }
    }/* if(FrameQBufMgr_module->refCount > 1) { */

     /* Release lock */
    Gate_leaveSystem(key);
    return (Startup_DONE);
}

/*
 *  ======== FrameQBufMgr_getInstNo ========
 *  API to get the unique instane no for the instances of FrmaeQBufMgr
 *  instances.
 */
Int32 FrameQBufMgr_getInstNo(UInt8 *instNo)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;
    UInt32 i;
    Int32  key;

    /* Get unique id for the instance */
    key = IGateProvider_enter (FrameQBufMgr_module->gate );
    for (i = 0; i < FrameQBufMgr_MAXINSTANCES; i++) {
         if (FrameQBufMgr_module->isInstNoValid[i] == FALSE) {
            FrameQBufMgr_module->isInstNoValid[i] = TRUE;
            /* found the  empty slot. This is the unique
             * instnace no for this instance on this processor
             *. By appending creator proc id we get the unique
              * instance id for this instance in the entire system
            */
            *instNo = i;
            break;
         }
    }
    IGateProvider_leave (FrameQBufMgr_module->gate, key);

    if (i == FrameQBufMgr_MAXINSTANCES) {
        status = FrameQBufMgr_E_MAXINSTANCES;
    }

    return (status);
}

/*
 *  ======== FrameQBufMgr_freeInstNo ========
 *  API to free the instance no to the FrameQBufMgr module.
 */
Int32 FrameQBufMgr_freeInstNo(UInt8 instNo)
{
    Int32  status = FrameQBufMgr_S_SUCCESS;
    Int32  key;

    if (instNo >= FrameQBufMgr_MAXINSTANCES) {
        status = FrameQBufMgr_E_FAIL;
    }
    else {
        /* Get unique id for the instance */
        key = IGateProvider_enter (FrameQBufMgr_module->gate );

        FrameQBufMgr_module->isInstNoValid[instNo] = FALSE;

        IGateProvider_leave (FrameQBufMgr_module->gate, key);
    }

    return (status);
}
/*
 *  ======== FrameQBufMgr_getBaseHeaderSize ========
 *  API to get the base frame heade if number of frame buffers known.
 */
UInt32 FrameQBufMgr_getBaseHeaderSize(UInt8 numFrameBufs)
{
    UInt32 baseHdrSize;

    Assert_isTrue ((numFrameBufs <= 8),
                    FrameQBufMgr_A_invalidParams);

    baseHdrSize = (  sizeof(FrameQBufMgr_FrameHeader)
                   + (sizeof(FrameQBufMgr_FrameBufInfo)* (numFrameBufs-1)));

    return (baseHdrSize);
}

/*
 *  ======== FrameQBufMgr_isSupportedInterface ========
 *  API to to find out the interface type is supported .
 */
Int32 FrameQBufMgr_isSupportedInterface(UInt32 type)
{
    Int32  status = TRUE;

    switch (type)
    {
        case FrameQBufMgr_INTERFACE_SHAREDMEM:
            /* Supported interface */
        break;
        default:
            /* Unsupported interface type specifiec */
            status = FALSE;
    }

    return (status);
}

/*!
 * @brief API to to find out the specified interface type is supported as buffer
 *  interface or not.
 * @param type  interface type.
 */
Int32 FrameQBufMgr_isSupportedBufInterface(UInt32 ctrlInterfaceType,UInt32 bufType)
{
    Int32  status = TRUE;

    switch (ctrlInterfaceType)
    {
        case FrameQBufMgr_INTERFACE_SHAREDMEM:

            switch (bufType)
            {
                /* HLOS side only FrameQBufMgr_BUFINTERFACE_SHAREDMEM is
                 * supported.
                 */
            case FrameQBufMgr_BUFINTERFACE_SHAREDMEM:
                /* Supported interface */
            case FrameQBufMgr_BUFINTERFACE_TILERMEM:
                /* Supported interface */
            break;
            default:
                /* unsupported buf interface type specified */
                status = FALSE;
            }

        break;
        default:
            /* unsupported interface type specifiec */
            status = FALSE;
    }

    return (status);
}

/*TBD The following API definations are temporary  hack.Need to remove*/
Ptr ti_syslink_ipc_rtos_FrameQBufMgr_add__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object
* obj, xdc_UInt8 arg1 ) {

    return NULL;
}
Int32 ti_syslink_ipc_rtos_FrameQBufMgr_alloc__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object * arg0, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame *arg1) {

    return -1;
}
Int32 ti_syslink_ipc_rtos_FrameQBufMgr_allocv__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object *arg0, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame *arg1, xdc_UInt32 *arg2, xdc_UInt8 *arg3) {
    return -1;
}
Int32 ti_syslink_ipc_rtos_FrameQBufMgr_dup__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object
*arg0, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame arg1, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame *arg2, xdc_UInt32 arg3) {
    return -1;
}
Int32 ti_syslink_ipc_rtos_FrameQBufMgr_dupv__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object
 *arg0, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame *arg1, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame ** arg2, xdc_UInt32 arg3, xdc_UInt32 arg4) {
    return -1;
}
Int32 ti_syslink_ipc_rtos_FrameQBufMgr_free__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object
 * arg0, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame arg1) {
    return -1;
}
Int32 ti_syslink_ipc_rtos_FrameQBufMgr_freev__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object * arg0, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame * arg1, xdc_UInt32 arg2) {
   return -1;
}
Int32 ti_syslink_ipc_rtos_FrameQBufMgr_invalidate__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object *arg0, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame arg1) {
    return -1;
}
Int32 ti_syslink_ipc_rtos_FrameQBufMgr_registerNotifier__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object *arg0, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_NotifyParams * arg1) {
    return -1;
}
Int32 ti_syslink_ipc_rtos_FrameQBufMgr_remove__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object *arg0, xdc_UInt8 arg1, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame arg2) {
    return -1;
}
Int32 ti_syslink_ipc_rtos_FrameQBufMgr_writeBack__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object *arg0, ti_syslink_ipc_rtos_interfaces_IFrameQBufMgr_Frame arg1) {
    return -1;
}

UInt32 ti_syslink_ipc_rtos_FrameQBufMgr_getId__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object * arg0) {
    return 0;
}

UInt32 ti_syslink_ipc_rtos_FrameQBufMgr_getBaseHeaderSize__F (UInt8 numFrameBufs)
{
    return FrameQBufMgr_getBaseHeaderSize(numFrameBufs);
}

Bool ti_syslink_ipc_rtos_FrameQBufMgr_isCacheEnabledForHeaderBuf__F (ti_syslink_ipc_rtos_FrameQBufMgr_Object *obj)
{
    return FrameQBufMgr_isCacheEnabledForHeaderBuf(obj);
}

Bool ti_syslink_ipc_rtos_FrameQBufMgr_isCacheEnabledForFrameBuf__F (ti_syslink_ipc_rtos_FrameQBufMgr_Object *obj,UInt8 frameBufIndex)
{
    return FrameQBufMgr_isCacheEnabledForFrameBuf(obj,frameBufIndex);
}

Int32 ti_syslink_ipc_rtos_FrameQBufMgr_getNumFreeFrames__F(ti_syslink_ipc_rtos_FrameQBufMgr_Object *obj,UInt32* numFreeFrames)
{
    return -1;
}

