/** 
 *  @file   FrameQBufMgr_ShMem.xdc
 *
 *  @brief      xdc file for FrameQBufMgr_ShMem.
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */





package ti.syslink.ipc.rtos.shMem;

import xdc.runtime.Error;
import xdc.runtime.IGateProvider;
import xdc.runtime.IHeap;
import ti.sdo.ipc.GateMP ;
import ti.sdo.utils.NameServer;
import xdc.runtime.IGateProvider;

@ModuleStartup
@InstanceInitError /* Initialization may throw errors */
@InstanceFinalize
module FrameQBufMgr_ShMem inherits ti.syslink.ipc.rtos.interfaces.IFrameQBufMgr {

    /*!
     *  Assert raised when an invalid parameter(s) is passed into a function
     */
    config xdc.runtime.Assert.Id A_invalidParams =
        {msg: "A_invalidParams: Invalid parameter"};

    /*!
     *  Assert raised when a null pointer needs to be dereferenced
     */
    config xdc.runtime.Assert.Id A_nullPointer =
        {msg: "A_nullPointer: Null pointer"};

   /*!
     *  Assert raised when null handle is returned from  modules.
     */
    config xdc.runtime.Assert.Id A_nullHandle =
        {msg: "A_nullHandle: Null Handle"};
    /*!
     *  Assert raised when Inst No of the instance is not valid.
     */
    config xdc.runtime.Assert.Id A_InvalidInstNo  = {
        msg: "A_InvalidInstNo: Invalid instance no"
    };

    /*!
     *  Assert raised when local protection provided is not sufficient for 
     * ClientNotifyMgr.
     */
    config xdc.runtime.Assert.Id A_localProtection  = {
        msg: "A_localProtection: Insufficient local protection "
    };

   /*!
     *  Assert raised when creator processor id of the instance is not valid.
     */
    config xdc.runtime.Assert.Id A_InvalidCreatorProcId  = {
        msg: "A_InvalidCreatorProcId: Invalid creator processor id"
    };

    /*!
     *  Assert raised when  invalid MemMgr type specified.
     */
    config xdc.runtime.Assert.Id A_invalid_MemMgrType  = {
        msg: "A_invalid_MemMgrType: Invalid MemMgrType"
    };
    /*!
     *  Error raised when invalid FrameQ_free is called.
     */
    config xdc.runtime.Assert.Id A_invalidFree  = {
        msg: "A_invalidFree: invalid FrameQ_free is called.Frame already freed"
    };
    /*!
     *  Error raised when invalid MemMgr type specified.
     */
    config Error.Id E_InvalidMemMgrType  = {
        msg: "E_InvalidMemMgrType: MemMgr type specified is invalid"
    };


    /*!
     *  Error raised when FrameQBufMgr is not created.
     */
    config Error.Id E_FrameQBufMgrNotCreated  = {
        msg: "E_FrameQBufMgrNotCreated: FrameQBufMgr instance is not created"
    };

    /*!
     *  Error raised when GateMP create failed.
     */
    config Error.Id E_GateMPCreate  = {
        msg: "E_GateMPCreate: GateMP create instance create failed"
    };

    /*!
     *  Error raised when GateMP open failed.
     */
    config Error.Id E_GateMPOpen  = {
        msg: "E_GateMPOpen: GateMP  instance open failed"
    };
    /*!
     *  Error raised when FrameQBufMgr is not Found in NameServer.
     */
    config Error.Id E_FrameQBufMgrNotFound  = {
        msg: "E_FrameQBufMgrNotFound: FrameQBufMgr instance is not found in NameServer"
    };

    
    /*!
     *  Error raised when ClientNotifyMgr Create failed.
     */
    config Error.Id E_ClientNotifyMgrCreate  = {
        msg: "E_ClientNotifyMgrCreate: ClientNotifyMgr instance Create Failed"
    };

    /*!
     *  Error raised when ClientNotifyMgrOpen failed.
     */
    config Error.Id E_ClientNotifyMgrOpen  = {
        msg: "E_ClientNotifyMgrOpen: ClientNotifyMgrOpen instance Open Failed"
    };

    /*!
     *  Error raised when invalid arguments passed to the create call.
     */
    config Error.Id E_InvArg  = {
        msg: "E_InvArg: Invalid arguments passed"
    };

    /*!
     *  Error raised when Insufficient shared memory provided to create call.
     */
    config Error.Id E_InvSize  = {
        msg: "E_InvSize: SharedMemory provided to create call is less than the instance shared memory requirements"
    };

    /*!
     *  Error raised when Insufficient shared addrprovided to create call.
     */
    config Error.Id E_InvAddr  = {
        msg: "E_InvAddr: Shared Addr provide to create call is NULL"
    };

    /*!
     *  Error raised when buffersize is not aligned to Cache line size.
     */
    config Error.Id E_CacheAlign  = {
        msg: "E_CacheAlign: BufferSize is not Cache Aligned"
    };
    /*!
     *  Error raised when max instances limit is reached
     */
    config Error.Id E_maxInstances  = {
        msg: "E_maxInstances: max instances limit reached"
    };
    /*!
     *  Error raised when postinitialize API failed 
     */
    config Error.Id E_postInitFail  = {
        msg: "E_postInitFail: Instance post init failed"
    };

    /*!
     *  Error raised when postinitialize API failed 
     */
    config Error.Id E_Fail  = {
        msg: "E_Fail: Api failed"
    };

    /*!
     *  Error raised when Memory_alloc  API failed 
     */
    config Error.Id E_MemoryAlloc  = {
        msg: "E_MemoryAlloc: Memory_alloc failed"
    };
    /*! 
     *  ======== OpenParams ========
     *  mouduleconfig structure.
     *
     *  @field(commonOpenParams)  Open params common to all the implementations.
     *                            see {@link CommonOpenParams}
     *
     */
    struct OpenParams {
        CommonOpenParams commonOpenParams;
    };
    /*!
     *  ======== open ========
     *  API to open the created instance and to get a handle to it.
     *
     *  The instance must be created usng create call before opening it.
     *
     *  @param(handlePtr)  Handle to the instance will be returned through this pointer.
     *  @param(name)       Name of the instance that needs to opened.ignored if name server is not configured.
     *
     *  @param(sharedAddr) Address of the shared control  info of the instance.
     *                     Must be passed if name server is not configured for
     *                     use by the module.
     *  @param(gate)       Gate for critical region protection.
     *
     *  @b(returns)        Returns positive value if success.
     */

    Int32  open(Handle *handlePtr, OpenParams *openParams);

    /*!
     *  ======== close ========
     *  Function to close the dynamically opened instance using {@link #open} call.
     *
     *
     *  @b(returns)                 returns 0 if  successfully registered the call back function.
     *
     */
     Int32 close(Handle * handlePtr);

    /*!
     *  ======== sharedMemInstReq ========
     *  Amount of shared memory required for creation of each instance
     *
     *  Can be used to make sure the {@link #sharedAddr} buffer is large
     *  enough before calling create.
     *
     *  All parameters except sharedAddr and sharedAddrSize must be
     *  the same that will be used in the create.
     *
     * @param(params)     Instance create/open parameters.
     * @param(frmHdrBufChunkSize) Buffer chunk size for frame headers.
     * @param(frmBufChunkSize) Buffer chunk size for framebuffers.
     *
     *
     *  @b(returns)       Amount of shared memory required
     */
    UInt32 sharedMemReq(const Params *params,
                        UInt32 *frmHdrBufChunkSize,
                        UInt32 *frmBufChunkSize);

    instance:
    /*!
     *  Create params  common to all the implementations of FrameQBufMgr.
     *
     */
     config CreateParams commonCreateParams = {size:0,ctrlInterfaceType:INTERFACE_NONE,name:null,openFlag:false};
    /*!
     *  Frame buffer params common to all types of allocations of buffers.
     *
     */
     config FrameBufParams  *frameBufParams[MAX_POOLS] = [null,null,null,null,null,null,null,null,
                                                          null,null,null,null,null,null,null,null,
                                                          null,null,null,null,null,null,null,null,
                                                          null,null,null,null,null,null,null,null,
                                                          null,null,null,null,null,null,null,null,
                                                          null,null,null,null,null,null,null,null,
                                                          null,null,null,null,null,null,null,null,
                                                          null,null,null,null,null,null,null,null
                                                         ];

    /*!
     *  IGateProvider used for critical region management of the shared memory
     *
     *  This must be a shared memory gate.
     */
    config GateMP.Handle gate = null;

    /*!
     *============localProtect ================
     * Default local protection level for the module instance.
     *
     */
    config  GateMP.LocalProtect localProtect = GateMP.LocalProtect_INTERRUPT;

    /*!
     *============remoteProtect ================
     * Default multiprocessor protection for the module instances.
     *
     */
    config  GateMP.RemoteProtect remoteProtect = GateMP.RemoteProtect_SYSTEM;

    /*!
     *  Alignment (in MAUs) of each block.
     *
     *  The alignment must be a power of 2. If the value 0 is specified,
     *  the value will be changed to meet the minimum structure alignment
     *  requirements (refer to
     *  {@link xdc.runtime.Memory#getMaxDefaultTypeAlign} and
     *  {@link xdc.runtime.Memory#getMaxDefaultTypeAlignMeta}),
     *  so the actual alignment may be larger.
     *
     *  For static creates, the FrameQBufMgr will allocate space for the buffer
     *  and will align the buffer on this requested alignment.
     *
     *  For dynamic creates, this parameter is used solely for error checking.
     *  The buffer provided to a dynamically created FrameQBufMgr must be aligned
     *  manually, and an assert will be raised if the buffer is not properly
     *  aligned. For dynamic creates, FrameQBufMgr will NOT adjust the buffer to
     *  satisfy the alignment.
     *
     *  The default alignment is 0.
     */
    config SizeT align = 0;

    /*! @_nodoc
     *  ======== sectionName ========
     *  Section name for the buffer managed by the instance.
     *
     *  The buffer allocated for a statically created FrameQBufMgr instance will be
     *  placed in this section.
     *
     *  The default section is the 'dataSection' in the platform.
     */
   metaonly config String sectionName = null;
    /*!
     *  ======== sharedAddr ========
     *  Virtual Address of the shared memory.Must be mapped
     *  on other processors if instance needs to be accessible
     *  from other processors.
     *
     *  The creator must supply the shared memory that
     *  this will use for maintain shared state information.
     *  Required parameter.
     */
    config Ptr sharedAddr = null;

    /*!
     *  ======== sharedAddrSize ========
     *  Size of shareAddr
     *
     *  Can use the {@link #sharedMemReq} or {@link #sharedMemReqMeta} call to
     *  determine the required size.
     *
     *  Required parameter.
     */
    config UInt32 sharedAddrSize = 0;

     /*!
     *  Shared region ID
     *
     *  The ID corresponding to the shared region in which this shared instance
     *  is to be placed.
     */
    config UInt32 regionId = 0;

     
     config UInt32  headerInterfaceType = INTERFACE_SHAREDMEM;
     config UInt32  bufInterfaceType = INTERFACE_SHAREDMEM;

     /*!
     *  ======== sharedAddrHdrBuf ========
     *  Virtual Address of the shared memory for frame header buffers.Must be mapped
     *  on other processors if the buffers managed by this instance needs to be
     *  accessible from other processors.
     *
     *  The creator can  supply the shared memory that this will use for
     *  maintaining the data buffers. Applications can create  instances without
     *  passing this address.In this case applications need to add buffers to
     *  the instance at runtime.
     */
    config    Ptr sharedAddrHdrBuf = null ;
    /*!
     *  ======== sharedAddrHdrBufSize ========
     *  Size of sharedAddrHdrBuf
     *
     */
    config UInt32 sharedAddrHdrBufSize = 0;

    /*!
     *  Shared region ID for Header buffers.
     *
     *  The ID corresponding to the shared region in From which Header buffers will be allocated
     * if sharedAddrHdrBuf is not provided.
     */
    config UInt32 hdrBuf_regionId = 0;

     /*!
      *  ======== sharedAddrFrmBuf ========
      *  Virtual shared Region Address of chunk  for frame buffers.
      */
     config    Ptr        sharedAddrFrmBuf = null;    

     /*!
      *  ======== sharedAddrFrmBufSize ========
      *  sharedAddrHdrBufSize Size of the sharedAddrFrmBuf.
      */
      config UInt32     sharedAddrFrmBufSize = 0;

    /*!
     *  Shared region ID for Frame buffers.
     *
     *  The ID corresponding to the shared region in From which Frame buffers will be allocated
     * if sharedAddrFrmBuf is not provided.
     */
    config UInt32 frmBuf_regionId = 0;


        
    /*!
     *  ======== numfreeFramePools ========
     *  Number of buffer pools.
     *
     *
     *  Required parameter.
     *
     *  The default number of free Frame pools is  0.
     */
    config  UInt32     numFreeFramePools    = 0;

    /*!
     *  ======== *numFreeFramesInPool ========
     *  UInt32 pointer pointing to Number of frames  in each pool.
     */
    config UInt32      *numFreeFramesInPool = null;
    /*!
     *  ======== numFreeHeaderBufsInPool ========
     *  UInt32 pointer pointing to Number of additional frame headers in 
     *  each pool.
     */
    config UInt32      *numFreeHeaderBufsInPool = null;
    /*!
     *  ======== frameHeaderBufSize ========
     *  UInt32 pointer pointing to headerBuffer sizes in 
     *  each pool.
     */
    config UInt32      *frameHeaderBufSize = null;
    /*!
     *  ======== numFrameBufsInFrame ========
     *  UInt32 pointer pointing to number of frame buffers in 
     *  each pool.
     */
    config UInt32      *numFrameBufsInFrame = null;

    /*!
     *  ======== frameBufSizes ========
     *  UInt32 pointer to pointer pointing to frame buf sizes of frame buffers in 
     *  each pool.
     */
    config UInt32      **frameBufSizes = null;

    /*!
     * Cache flags denoting whether cache coherence operations are  needed to be performed on instance shared control structures.
     */
    config   UInt32 cpuAccessFlags = 0 ;
    /*!
     * How many clients that do reister  callback function with the instance.
     */
    config  UInt32 numNotifyEntries = 1;
    internal:

    Int32 postInit(Object *obj,const Params* params,Error.Block *eb ) ;
    /*!
     *  ======== Module_State ========
     *  Module  state Object structure.
     *
     *  @field(nameServer)        Handle to the created NameServer instance for this HeapNotifyMutiBufSM module.
     *  @field(gate)              gat for the module wide protection on this processor.
     *  @field(gateType)          Stattically created gate or dynamically created gate. 
     *  @field(refCount)          Module initialization count.     
     *  @field(isInstNoValid)     Create call will check this flags and sets first invalid index to valid
     *                            and  assigns this index to the first byte of obj->Id as instNo.
     *
     */

    struct Module_State {
        NameServer.Handle     nameServer;
        IGateProvider.Handle  gate ;
        UInt32                gateType;
        UInt32                refCount;
        Ptr                   defaultInstGate;
        Config                cfg;
        Bool                  isInstNoValid[];
    };

    /*!
     *  ======== Instance_State ========
     *  Instance  state Object structure.
     *  @field(nsKey)            Holds return value of Nameserver_add(for removing NS entry). valid
     *                           for creator only.
     *  @field(regionId)         Shared region index from where instance control
     *                           structure is allocated from.
     *  @field(minAlign)         Alignment of the controls structures. 
     *                           usually equal to cacheline size of the shared region.
     *  @field(allocSize)        Size of the memory allocated  for the instance
     *                           control structure from the shmIndex.
     *  @field(gate)             Gate for critical region management.
     *  @field(creGate)          Indicates whether gate is created internally.
     *  @field(gateMPAllocSize)  Indicates size of the allocated memory for the
     *                           instance GateMP if it is created internally.
     *  @field(instGateSharedAddr)Indicates sharedAddr of the allocated memory
     *                            for the instance GateMP if it is created
     *                            internally.Valid for creator only.
     *  @field(exactMatchReq)    Flag indicating whether this client requires exact sized buffer or not.
     *  @field(isRegistered)     Flag to indicate whether this client has registed a notification function
     *                           with ClientNotifyMgr.
     *  @filed(Id)               Id of the instance.
     *  @field(notifyId)         Varible  to hold the  index returned by ClientNotifyMgr_registerClient.
     *  @filed(objType)          Indicates type of object. Dynamic create or Dynamic open etc.
     *  @field(pnameserverEntry) Pointer to the namserver info located the start
     *                           of the instance shared Addr.
     *  @field(attrs)            Pointer to the shared instance attribute 
     *                           structure.
     *  @field(clientNotifyMgrHandle) Handle to the ClientNotifyMgr instance created for this FrameQBufMgr instance.
     *  @field(nameServerName)   Name of the NameServer instance .
     *  @field(creCliGate)   To keep track of if clientnotifymgr gate is created internally .
     *
     */
    /* not for client use */
     struct Instance_State {
        Ptr                     nsKey;
        UInt32                  regionId;       /* Region id of the shared region */
        SizeT                   allocSize;      /* Shared memory allocated       */
        UInt32                  hdrBuf_regionId;
        Ptr                     hdrBufPtr;
        SizeT                   hdrBufAllocSize;
        UInt32                  frmBuf_regionId;
        Ptr                     frmBufPtr;
        SizeT                   frmBufAllocSize;
        /*GateMP.Handle           gate ;*/
        Ptr                     gate ;
        Bool                    creGate;
        UInt32                  gateMPAllocSize;
        Ptr                     instGateSharedAddr;
        UInt32                  instId ;
        UInt32                  notifyId ;
        UInt32                  objType ;
        UInt32                  ctrlInterfaceType;
        UInt32                  headerInterfaceType;
        UInt32                  bufInterfaceType;
        UInt32                     minAlign;
        Ptr                     pnameserverEntry;
        volatile   Attrs        *attrs;
        volatile   FreeFramePoolObj **freeFramePool;
        UInt32                  cliGateMPShAddrSize;
        Ptr                     clientNotifyMgrHandle;
       /* GateMP.Handle           clientNotifyMgrGate;*/
        Ptr                     clientNotifyMgrGate;
        UInt32                  refCount;
        UInt32                  memMgrType_HdrBufs;
        Ptr                     memMgrAllocater_hdrBufs;
        IHeap.Handle            heapMemMPHandle_HdrBufs;
        UInt32                  memMgrType_FrmBufs;
        Ptr                     memMgrAllocater_frmBufs;
        IHeap.Handle            heapMemMPHandle_FrmBufs;
        Char                    name[32];
        Bool                    ctrlStructCacheFlag ;
        Bool                    frmHdrBufCacheFlag ;
        Bool                    frmBufCacheFlag[8] ;
        Bool                    bufCpuAccessFlag[8];
        Bool                    isRegistered ;
        Bool                    creCliGate;
     };

}
